-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, Attend = ...
Attend.name = "Attend"
Attend.slashName = "attend"
Attend.version = 2.1
Attend.startup = false
Attend.debug = false
Attend.raidPlayerLimit = 9
Attend.raidsToKeep = 20
Attend.ignoreCrossShardRaids = true
Attend.maxLevel = 60
Attend.minRaidLevel = 60
Attend.x = 200
Attend.y = 200
Attend.buttonX = 200
Attend.buttonY = 200
Attend.raidListRowLayout = nil
Attend.raidDetailRowLayout = nil
Attend.refreshOldCount = 0
Attend.raidZoneMap = {
	["Warboss Drak"] = "ID",
	["Ituziel"] = "ID",
	["Ember Conclave"] = "ID",
	["Maklamos the Scryer"] = "ID",
	["Rusila Dreadblade"] = "ID",
	["Murdantix"] = "HK",
	["Matron Zamira"] = "HK",
	["Soulrender Zilas"] = "HK",
	["Vladmal Prime"] = "HK",
	["Sicaron"] = "HK",
	["King Molinar"] = "HK",
	["Prince Dollin"] = "HK",
	["Estrode"] = "HK",
	["Grugonim"] = "HK",
	["Inquisitor Garau"] = "HK",
	["Aqualix"] = "HK",
	["Denizar"] = "HK",
	["Inwar Darktide"] = "HK",
	["Jornaru"] = "HK",
	["Akylios"] = "HK",
	["Ereandorn"] = "ROTP",
	["Beruhast"] = "ROTP",
	["General Silgen"] = "ROTP",
	["High Priest Arakhurn"] = "ROTP",
	["Dark Focus"] = "ROS",
	["Warmaster Galenir"] = "ROS",
	["Plutonus the Immortal"] = "ROS",
	["Herald Gaurath"] = "ROS",
	["Alsbeth the Discordant"] = "ROS",
	["Duke Letareus"] = "GSB",
	["Infiltrator Johlen"] = "GSB",
	["Oracle Aleria"] = "GSB",
	["Prince Hylas"] = "GSB",
	["Lord Greenscale"] = "GSB",
	["Anrak the Foul"] = "GP",
	["Guurloth"] = "GP",
	["Thalguur"] = "GP",
	["Uruluuk"] = "GP",
	["Assault Commander Jorb"] = "DH",
	["Joloral Ragetide"] = "DH",
	["Isskal"] = "DH",
	["High Priestess Hydriss"] = "DH",
	["Cyril"] = "TDQ",
	["General Typhiria"] = "TDQ",
	["Grand Falconer Zoles"] = "TDQ",
	["Jultharin"] = "TDQ",
	["Dread Lord Goloch"] = "EE",
	["Kain the Reaper"] = "EE",
	["Matriarch of Pestilence"] = "EE",
	["Progenitor Saetos"] = "EE",
	["Regulos"] = "EE",
	["Artifex Zaviel"] = "FT",
	["Crucia"] = "FT",
	["Gangnum"] = "FT",
	["Gelidra"] = "FT",
	["Kolmsasveli"] = "FT",
	["Toinenveli"] = "FT",
	["Ultane"] = "GA",
	["Salvarola"] = "GA",
	["Kyzan"] = "GA",
	["Magma Lord Kaliban"] = "GA",
	["Proteus"] = "PBB",
	["Krizzix"] = "PBB",
	["Warden Thrax"] = "PBB",
	["Inyr'Kta"] = "PBB",
	["Planebreaker Abominus"] = "PBB",
	["Breaker X-1"] = "IG",
	["Mordan"] = "IG",
	["Viktus"] = "IG",
	["Volan"] = "IG"
}

local raids = {}
local currentRaid = {}
local raidWindowButtonMap = {}
local raidDetail = {}
local raidLoot = {}
local raidPlayer = ""
local raidItem = ""
local raidToDelete = nil
local raidToEnd = nil

--
-- Main functions
--

function Attend.Debug(str)
	if Attend.debug == true then
		print(str)
	end
end

function Attend.GetPlayerDescription(player)
	local ret = string.format("Player: %s %s", player.name, Attend.FormatTime(player.duration))
	if player.standbyDuration > 0 then
		ret = ret .. string.format(" (standby %s)", Attend.FormatTime(player.standbyDuration))
	end
	return ret
end

function Attend.GetTimeString(timeVal, xmlFormat)
	local timeStr = ""
	local formatStr = "%04d-%02d-%02d %02d:%02d:%02d"
	if xmlFormat == true then
		formatStr = "%04d-%02d-%02dT%02d:%02d:%02d"
	end
	
	if os ~= nil and os.date ~= nil and timeVal ~= nil then
		local dateFields = os.date("*t", timeVal)
		if dateFields.year ~= nil and dateFields.year > 2010 then
			timeStr = string.format(formatStr, dateFields.year, dateFields.month,
									dateFields.day, dateFields.hour, dateFields.min, dateFields.sec)
		end
	end
	
	return timeStr
end

function Attend.GetRaidDescriptionDetails(thisRaid, xmlFormat)
	local startString = Attend.GetTimeString(thisRaid.startTime, xmlFormat)
	local durationString = "Ongoing"
	if thisRaid.endTime then
		durationString = string.format("%s", Attend.FormatTime(thisRaid.endTime - thisRaid.startTime))
	end
	
	local playerNames = {}
	for n in pairs(thisRaid.players) do
		table.insert(playerNames, n)
	end
	
	table.sort(playerNames)
	
	local tempLocation = Attend.GetRaidLocation(thisRaid)
	if tempLocation == "" then
		tempLocation = "RAID"
	end
	
	return tempLocation, startString, durationString, playerNames
end

function Attend.GetRaidDescription(thisRaid)
	local tempLocation, startString, durationString, playerNames = Attend.GetRaidDescriptionDetails(thisRaid)
	return string.format("%s(%s): %s, %d participants", tempLocation, startString, durationString, #playerNames), playerNames
end

function Attend.GetRaidLocation(thisRaid)
	if thisRaid.location == nil then
		thisRaid.location = ""
	end
	
	return thisRaid.location
end

function Attend.GetBossKills(thisRaid)
	if thisRaid.bossKills == nil then
		thisRaid.bossKills = {}
	end

	return thisRaid.bossKills
end

function Attend.DumpRaid(raidName, thisRaid)
	local raidDesc, playerNames = Attend.GetRaidDescription(thisRaid)
	
	print(raidDesc)
    	for i,n in ipairs(playerNames) do
    		print("    " .. Attend.GetPlayerDescription(thisRaid.players[n]))
    	end
end

function Attend.Dump()
	print("Attendance dump start-----------------");
	
	for indx = 1, #raids do
		Attend.DumpRaid(string.format("%d", indx), raids[indx]);
	end
	
	if Attend.IsInRaid() then
		Attend.DumpRaid("current", currentRaid)
	end

	print("Attendance dump end-------------------");
end

function Attend.IsInRaid()
	return currentRaid.startTime
end

function Attend.UpdatePlayerDuration(player, currentTime)
	if player.startTime then
		player.duration = player.duration + (currentTime - player.startTime)
		if not player.maxEndTime or player.maxEndTime < currentTime then
			player.maxEndTime = currentTime
		end
		
		player.startTime = nil
	end
	
	if player.standbyStartTime then
		player.standbyDuration = player.standbyDuration + (currentTime - player.standbyStartTime)
		player.standbyStartTime = nil
	end
end

function Attend.CreateEmptyRaid()
	Attend.Debug("Create EmptyRaid")
	currentRaid = {}
	currentRaid.startTime = nil
	currentRaid.endTime = nil
	currentRaid.players = {}
	currentRaid.location = ""
	currentRaid.bossKills = {}
end

function Attend.StartRaid(currentTime)
	Attend.Debug("StartRaid " .. Attend.GetTimeString(currentTime, false))
	currentRaid.startTime = currentTime
	currentRaid.endTime = nil
	currentRaid.players = {}
	
	currentRaid.playerName = Attend.playerName
end

function Attend.TrimRaids()
	-- Trim our raid list
	while(#raids > Attend.raidsToKeep) do
		table.remove(raids, 1)
	end
end

function Attend.EndRaid(currentTime)
	if currentRaid.startTime then
		Attend.Debug("EndRaid " .. Attend.GetTimeString(currentRaid.startTime, false) .. " " .. Attend.GetTimeString(currentTime, false))
		currentRaid.endTime = currentTime

		-- Total up all the player durations
		for playerName, player in pairs(currentRaid.players) do
			Attend.UpdatePlayerDuration(player, currentTime)
		end

		-- Add this raid to our saved list
		raids[#raids + 1] = currentRaid

		Attend.TrimRaids()

		Attend.CreateEmptyRaid()
	end
end

function Attend.GetPlayerNames()
	local players = {}
	local playerCount = 0
	local crossShard = false

	for id = 1, 20 do
		local unitSpecifier = string.format("group%02d", id)
		unitdetail = Inspect.Unit.Detail(unitSpecifier)
		if unitdetail then
			if string.find(unitdetail.name, "@") then
				crossShard = true
			end
			
			if unitdetail.level and type(unitdetail.level) == "number" and unitdetail.level < Attend.minRaidLevel then
				return 0, {}, false
			end
			
			local playerName = string.gsub(unitdetail.name, "@.*$", "")
			
			local playerClass = "unknown"
			if unitdetail.calling then
				playerClass = unitdetail.calling
			end
			
			players[playerName] = playerClass
			playerCount = playerCount + 1
		end
	end
	
	return playerCount, players, crossShard
end

function Attend.CreateNewPlayer(playerName, playerClass, currentTime)
	local player = {}
	player.name = playerName
	if playerClass ~= "unknown" then
		player.class = playerClass
	end
	player.startTime = currentTime
	player.minStartTime = currentTime
	player.duration = 0
	player.standbyStartTime = nil
	player.standbyDuration = 0
	player.loot = {}
	player.points = {}
	return player
end

function Attend.AddNewPlayers(players, currentTime)
	for playerName, playerClass in pairs(players) do
		local player = currentRaid.players[playerName]
		if not player then
			currentRaid.players[playerName] = Attend.CreateNewPlayer(playerName, playerClass, currentTime)
		elseif not player.startTime then
			player.startTime = currentTime
			if player.standbyStartTime then
				player.standbyDuration = player.standbyDuration + (currentTime - player.standbyStartTime)
				player.standbyStartTime = nil
			end
		elseif not player.class then
			if playerClass ~= "unknown" then
				player.class = playerClass
			end
		end
	end
end

function Attend.RemoveOldPlayers(players, currentTime)
	for playerName, player in pairs(currentRaid.players) do
		if not players[playerName] then
			Attend.UpdatePlayerDuration(player, currentTime)
		end
	end
end

function Attend.AddStandbyPlayer(givenName, currentTime)
	if not givenName or string.len(givenName) < 1 then
		return "You must supply a player name to add a standby";
	end
	
	local playerName = Attend.CapitaliseName(givenName)
	
	if not Attend.IsInRaid() then
		return string.format("You must be in a raid with at least %d members before adding standby players", Attend.raidPlayerLimit);
	end
	
	local playerCount, players, crossShard = Attend.GetPlayerNames()
	
	if not players[playerName] then
		local player = {}
		player.name = playerName
		player.startTime = nil
		player.duration = 0
		player.standbyStartTime = currentTime
		player.standbyDuration = 0
		currentRaid.players[playerName] = Attend.CreateNewPlayer(playerName, "unknown", currentTime)
		return string.format("Player %s added as standby", playerName);
	else
		return string.format("Player %s is already in the raid and cannot be standby", playerName);
	end
end

function Attend.GetPlayerLoot(thisRaid, playerName)
	if not thisRaid then
		print("Cannot add loot to a nil raid")
		return
	end

	if not playerName then
		print("Cannot add loot to a nil player")
		return
	end

	if not thisRaid.players[playerName] then
		print(string.format("Cannot find player %s in current raid to assign loot", playerName))
		return
	end
	
	if not thisRaid.players[playerName].loot then
		thisRaid.players[playerName].loot = {}
	end
	
	return thisRaid.players[playerName].loot
end

function Attend.EditLoot(thisRaid, playerName, lootNumber, itemName, itemCost, bossName)
	local playerLoot = Attend.GetPlayerLoot(thisRaid, playerName)

	if not itemName or string.len(itemName) == 0 then
		print("Cannot add empty loot")
		return
	end
	
	if not type(itemCost) == "number" then
		print("Item cost must be a number")
		return
	end
	
	if lootNumber < 1 or lootNumber > #playerLoot then
		print("Invalid loot item number " .. lootNumber)
		return
	end
	
	local loot = playerLoot[lootNumber]
	loot.name = itemName
	loot.cost = itemCost
	loot.bossName = bossName
end

function Attend.AddLoot(thisRaid, playerName, itemName, itemCost, bossName)
	local playerLoot = Attend.GetPlayerLoot(thisRaid, playerName)

	if not itemName or string.len(itemName) == 0 then
		print("Cannot add empty loot")
		return
	end
	
	if not type(itemCost) == "number" then
		print("Item cost must be a number")
		return
	end
		
	local loot = {}
	loot.name = itemName
	loot.cost = itemCost
	loot.bossName = bossName
	
	table.insert(playerLoot, loot)
end

function Attend.RemoveLoot(thisRaid, playerName, lootNumber)
	local playerLoot = Attend.GetPlayerLoot(thisRaid, playerName)
	table.remove(playerLoot, lootNumber)
end

function Attend.AddPlayerLoot(args)
	local arg = Attend.SplitArgs(args)
	
	if #arg < 4 then
		return "Usage: attend loot playername itemname cost"
	end
	
	local itemCost = tonumber(arg[#arg])
	if not itemCost then
		return "Last parameter to attend loot must be the item cost as a number"
	end
	
	local thisRaid = currentRaid
	
	local playerName = Attend.CapitaliseName(arg[2])
	if not thisRaid then
		return "Cannot add loot when not in a raid"
	end
	
	if not thisRaid.players[playerName] then
		return "Player " .. playerName .. " is not in the current raid"
	end
	
	local itemName = arg[3]
	for indx = 4, #arg - 1 do
		itemName = itemName .. " " .. arg[indx]
	end
	
	local itemLen = string.len(itemName)
	if string.sub(itemName, 1, 1) == "[" and string.sub(itemName, itemLen, itemLen) == "]" then
		itemName = string.sub(itemName, 2, itemLen - 1)
	end
	
	Attend.AddLoot(thisRaid, playerName, itemName, itemCost, "boss")
	
	return "Added item [" ..  itemName .. "] to player " .. playerName .. " for a cost of " .. itemCost;
end

function Attend.Refresh()
	local playerCount, players, crossShard = Attend.GetPlayerNames()
	if playerCount ~= Attend.refreshOldCount then
		Attend.Debug("Refresh " .. playerCount .. " " .. tostring(crossShard))
		Attend.refreshOldCount = playerCount
	end
	
	local currentTime = Attend.GetTime()
	
	if Attend.ignoreCrossShardRaids and crossShard then
		if Attend.IsInRaid() then
			Attend.CreateEmptyRaid()
		end
	else
		if playerCount >= Attend.raidPlayerLimit then
			if not Attend.IsInRaid() then
				Attend.StartRaid(currentTime)
			end

			Attend.AddNewPlayers(players, currentTime)
			Attend.RemoveOldPlayers(players, currentTime)
		else
			-- We are no longer in a raid, so close the current raid
			Attend.EndRaid(currentTime)
		end
	end
end

function Attend.SplitArgs(str)
	local result = {}
	local count = 1
	for token in string.gmatch(str, "[^%s]+") do
		result[count] = token
		count = count + 1
	end
	return result
end

function Attend.GetPlayerBBCode(thisRaid, n)
	local bbCode = string.format("[*]%s (%s)", thisRaid.players[n].name, Attend.FormatTime(thisRaid.players[n].duration))
	if thisRaid.players[n].standbyDuration > 0 then
		bbCode = bbCode .. string.format(" (standby %s)", Attend.FormatTime(thisRaid.players[n].standbyDuration))
	end
	bbCode = bbCode .. "\n"

	local lootStr = ""
	local hasLoot = false
	local loot = Attend.GetPlayerLoot(thisRaid, n)
	for _, lootDetail in ipairs(loot) do
		lootStr = lootStr .. "[*]" .. n .. ": [item]" .. lootDetail.name .. "[/item] (" .. lootDetail.cost .. ")"
		if lootDetail.bossName and lootDetail.bossName ~= "boss" then
			lootStr = lootStr .. " from " .. lootDetail.bossName
		end
		lootStr = lootStr .. "\n"
		hasLoot = true
	end
	
	return bbCode, hasLoot, lootStr
end

function Attend.GetPlayerXML(thisRaid, n)
	local xml = "    <Player>\n"
	xml = xml .. "      <Name>" .. thisRaid.players[n].name .. "</Name>\n"
	if thisRaid.players[n].class ~= nil then
		xml = xml .. "      <Class>" .. thisRaid.players[n].class .. "</Class>\n"
	else
		xml = xml .. "      <Class>unknown</Class>\n"
	end
	
	if thisRaid.players[n].minStartTime and thisRaid.players[n].maxEndTime then
		xml = xml .. "      <StartTime>" .. Attend.GetTimeString(thisRaid.players[n].minStartTime, true) .. "</StartTime>\n"
		xml = xml .. "      <EndTime>" .. Attend.GetTimeString(thisRaid.players[n].maxEndTime, true) .. "</EndTime>\n"
	end
	
	xml = xml .. "      <Duration>" .. Attend.FormatTime(thisRaid.players[n].duration) .. "</Duration>\n"
	
	if thisRaid.players[n].standbyDuration > 0 then
		xml = xml .. "      <Standby>" .. Attend.FormatTime(thisRaid.players[n].standbyDuration) .. "<Standby>\n"
	end

	local lootStr = "      <Loot>\n"
	local hasLoot = false
	local loot = Attend.GetPlayerLoot(thisRaid, n)
	for _, lootDetail in ipairs(loot) do
		lootStr = lootStr .. "        <Item>\n"
		lootStr = lootStr .. "          <Name>" .. lootDetail.name .. "</Name>\n"
		lootStr = lootStr .. "          <Cost>" .. lootDetail.cost .. "</Cost>\n"
		if lootDetail.bossName and lootDetail.bossName ~= "boss" then
			lootStr = lootStr .. "          <Boss>" .. lootDetail.bossName .. "</Boss>\n"
		end
		lootStr = lootStr .. "        </Item>\n"
		hasLoot = true
	end
	
	lootStr = lootStr .. "      </Loot>\n"
	
	if hasLoot == true then
		xml = xml .. lootStr
	end
	
	xml = xml .. "    </Player>\n"
	
	return xml
end

function Attend.GetBBCodeExport(thisRaid)
	local bbCode = ""
	
	local raidDesc, playerNames = Attend.GetRaidDescription(thisRaid)
	
	local durationString = "Ongoing"
	if thisRaid.endTime then
		durationString = string.format("%s", Attend.FormatTime(thisRaid.endTime - thisRaid.startTime))
	end
	
	local playerClasses = {}
	local playerClassExists = {}
	for _, n in pairs(thisRaid.players) do
		if n.class and not playerClassExists[n.class] then
			playerClassExists[n.class] = 1
			table.insert(playerClasses, n.class)
		end
	end
	
	table.sort(playerClasses)

    local lootStr = "[b]Loot:[/b]\n[list]"
    local hasLoot = false
	bbCode = bbCode .. string.format("[b]%s[/b]\n[list]", raidDesc)
	for _, c in ipairs(playerClasses) do
		bbCode = bbCode .. c .. "\n[list]"
		for i,n in ipairs(playerNames) do
			if thisRaid.players[n].class and thisRaid.players[n].class == c then
				local playerBBCode, playerHasLoot, playerLootStr = Attend.GetPlayerBBCode(thisRaid, n)
				bbCode = bbCode .. playerBBCode
				hasLoot = hasLoot or playerHasLoot
				if playerHasLoot == true then
					lootStr = lootStr .. playerLootStr
				end
			end
		end
	    	bbCode = bbCode .. "[/list]\n"
	end
	
	local unknownPlayers = false
	for i,n in ipairs(playerNames) do
		if not thisRaid.players[n].class then
			if unknownPlayers == false then
				unknownPlayers = true
				bbCode = bbCode .. "unknown\n[list]"
			end
			
			local playerBBCode, playerHasLoot, playerLootStr = Attend.GetPlayerBBCode(thisRaid, n)
			bbCode = bbCode .. playerBBCode
			hasLoot = hasLoot or playerHasLoot
			if playerHasLoot == true then
				lootStr = lootStr .. playerLootStr
			end
		end
	end
	
	if unknownPlayers == true then
		bbCode = bbCode .. "[/list]\n"
	end
	
	bbCode = bbCode .. "[/list]\n"

	if hasLoot then
    	lootStr = lootStr .. "[/list]\n"
		bbCode = bbCode .. lootStr
	end
	
	local hasDeaths = false
	local deathStr = "[b]Boss Kills:[/b][list]"
	for name, death in pairs(Attend.GetBossKills(thisRaid)) do
		hasDeaths = true
		deathStr = deathStr .. "[*]" .. name .. " - " .. Attend.GetTimeString(death) .. "\n"
	end
	
	if hasDeaths then
    	deathStr = deathStr .. "[/list]\n"
		bbCode = bbCode .. deathStr
	end
	
	return bbCode
end

function Attend.GetXMLExport(thisRaid)
	local xml = "<Raid>\n"
	
	local tempLocation, startString, durationString, playerNames = Attend.GetRaidDescriptionDetails(thisRaid, true)
	xml = xml .. "  <Location>" .. tempLocation .. "</Location>\n"
	xml = xml .. "  <Started>" .. startString .. "</Started>\n"
	xml = xml .. "  <Duration>" .. durationString .. "</Duration>\n"
	
	xml = xml .. "  <Players>\n"
	for i,n in ipairs(playerNames) do
		local playerxml = Attend.GetPlayerXML(thisRaid, n)
		xml = xml .. playerxml
	end
		
	xml = xml .. "  </Players>\n"

	local hasDeaths = false
	local deathStr = "  <BossKills>\n"
	for name, death in pairs(Attend.GetBossKills(thisRaid)) do
		hasDeaths = true
		deathStr = deathStr .. "    <Boss>\n"
		deathStr = deathStr .. "      <Name>" .. name .. "</Name>\n"
		deathStr = deathStr .. "      <DeathTime>" .. Attend.GetTimeString(death, true) .. "</DeathTime>\n"
		deathStr = deathStr .. "    </Boss>\n"
	end
	
	if hasDeaths then
		deathStr = deathStr .. "  </BossKills>\n"
		xml = xml .. deathStr
	end
	
	xml = xml .. "</Raid>\n"
	return xml
end

function Attend.FormatTime(secs)
	local hours = math.floor(secs / 3600);
	local mins = math.floor((secs % 3600) / 60);
	local seconds = math.floor(secs % 60);
	return string.format("%02d:%02d:%02d", hours, mins, seconds);	
end

function Attend.GetTime()
	if os ~= nil and os.time ~= nil then
		return os.time()
	else
		return Inspect.Time.Frame()
	end
end

function Attend.MergeRaids(raid1, raid2)
	if not raid1.endTime or not raid2.endTime then
		print("You can only merge two completed raids")
		return nil
	end
	
	local newRaid = {}

	newRaid.startTime = raid1.startTime
	if raid2.startTime < newRaid.startTime then
		newRaid.startTime = raid2.startTime
	end
	
	newRaid.endTime = raid1.endTime
	if raid2.endTime > newRaid.endTime then
		newRaid.endTime = raid2.endTime
	end

	newRaid.playerName = raid1.playerName	
	newRaid.players = raid1.players

	for name, player in pairs(raid2.players) do
		if newRaid.players[name] then
			newRaid.players[name].duration = newRaid.players[name].duration + player.duration
			newRaid.players[name].standbyDuration = newRaid.players[name].standbyDuration + player.standbyDuration
			
			local playerLoot = Attend.GetPlayerLoot(newRaid, name)
			local playerLoot2 = Attend.GetPlayerLoot(raid2, name)
			for _, loot in ipairs(playerLoot2) do
				table.insert(playerLoot, loot)
			end
		else
			newRaid.players[name] = player
		end
	end
	
	newRaid.location = Attend.GetRaidLocation(raid1)
	newRaid.bossKills = Attend.GetBossKills(raid1)
	
	local tempLocation = Attend.GetRaidLocation(raid2)
	if newRaid.location ~= "" and tempLocation ~= "" then
		newRaid.location = newRaid.location .. " & " .. raid2.location
	elseif tempLocation ~= "" then
		newRaid.location = tempLocation
	end
	
	for name, death in pairs(Attend.GetBossKills(raid2)) do
		newRaid.bossKills[name] = death
	end
	
	return newRaid
end

function Attend.DeleteRaid(thisRaid)
	for indx = 1, #raids do
		if raids[indx] == thisRaid then
			table.remove(raids, indx)
			break
		end
	end
end

function Attend.GetNumber(value)
	local ret = tonumber(value)
	if not ret then
		ret = 0
	end
	
	return ret
end

function Attend.CapitaliseName(str)
	return str:gsub("^%l", string.upper)
end

--
-- User Interface (UI)
--

function Attend.ConfigWindowClose(makeChanges)
	if makeChanges then
		Attend.ignoreCrossShardRaids = Attend.ignoreCrossShardCheck:GetChecked()
		Attend.raidPlayerLimit = Attend.NumberSlider.GetValue(Attend.raidPlayerLimitSlider)
		Attend.raidsToKeep = Attend.NumberSlider.GetValue(Attend.raidsToKeepSlider)
		Attend.minRaidLevel = Attend.NumberSlider.GetValue(Attend.minRaidLevelSlider)
		Attend.TrimRaids()
	end
	
	Attend.configWindow:SetVisible(false)
	Attend.SetRaidListWindowEnabled(true)
end

function Attend.CreateConfigWindow(parentFrame, positonFrame)
	Attend.configWindow = UI.CreateFrame("SimpleWindow", "ConfigWindow", parentFrame)
	Attend.configWindow:SetVisible(false)
	Attend.configWindow:SetTitle("Configuration window")
	Attend.configWindow:SetHeight(300)
	Attend.configWindow:SetPoint("CENTER", positonFrame, "CENTER")
	Attend.configWindow:SetLayer(5)
	
	local rowFrame = UI.CreateFrame("Frame", "RaidDetailAttend.RowLayout", Attend.configWindow)
	rowFrame:SetPoint("TOPLEFT", Attend.configWindow, "TOPLEFT", 50, 70)
	rowFrame:SetPoint("BOTTOMRIGHT", Attend.configWindow, "BOTTOMRIGHT", -50, -70)
	
	Attend.configAttendRowLayout = Attend.RowLayout.Create(rowFrame, 4, 2, 40, 70, 2, 2)
	Attend.RowLayout.SetColWidth(Attend.configAttendRowLayout, 1, 260)

	local labelFrame1 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 1, 1, "Text")
	labelFrame1:SetText("Number of raids to keep")
	local numberFrame1 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 1, 2, "Frame")
	Attend.raidsToKeepSlider = Attend.NumberSlider.Create(numberFrame1, 1, 50, 1, 1)

	local labelFrame2 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 2, 1, "Text")
	labelFrame2:SetText("Only record raids with size above")
	local numberFrame2 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 2, 2, "Frame")
	Attend.raidPlayerLimitSlider = Attend.NumberSlider.Create(numberFrame2, 2, 20, 1, 2)

	local labelFrame3 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 3, 1, "Text")
	labelFrame3:SetText("Ignore cross shard raids (i.e. wf's)")

	local checkFrame1 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 3, 2, "Frame")
	Attend.ignoreCrossShardCheck = UI.CreateFrame("RiftCheckbox", "IgnoreCheckbox", checkFrame1)
	Attend.ignoreCrossShardCheck:SetPoint("TOPLEFT", Attend.ignoreCrossShardCheck:GetParent(), "TOPLEFT")

	local labelFrame4 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 4, 1, "Text")
	labelFrame4:SetText("Only record raids with minimum character level")
	local numberFrame4 = Attend.RowLayout.CreateFrame(Attend.configAttendRowLayout, 4, 2, "Frame")
	Attend.minRaidLevelSlider = Attend.NumberSlider.Create(numberFrame4, 1, Attend.maxLevel, 1, Attend.minRaidLevel)

	Attend.configOKButton = UI.CreateFrame("RiftButton", "ConfigOKButton", Attend.configWindow)
	Attend.configOKButton:SetText("OK")
	Attend.configOKButton:SetPoint("BOTTOMLEFT", Attend.configWindow, "BOTTOMLEFT", 20, -20)
	Attend.configOKButton.Event.LeftPress = function() Attend.ConfigWindowClose(true) end
	
	Attend.configCancelButton = UI.CreateFrame("RiftButton", "ConfirmationCancelButton", Attend.configWindow)
	Attend.configCancelButton:SetText("Cancel")
	Attend.configCancelButton:SetPoint("BOTTOMRIGHT", Attend.configWindow, "BOTTOMRIGHT", -20, -20)
	Attend.configCancelButton.Event.LeftPress = function() Attend.ConfigWindowClose(false) end
	
	Attend.configWindow:SetWidth(Attend.configOKButton:GetWidth() * 2 + 160)

	Attend.RowLayout.Redraw(Attend.configAttendRowLayout)
end

function Attend.ShowConfigWindow()
	Attend.SetRaidListWindowEnabled(false)
	Attend.ignoreCrossShardCheck:SetChecked(Attend.ignoreCrossShardRaids)
	Attend.NumberSlider.SetValue(Attend.raidPlayerLimitSlider, Attend.raidPlayerLimit)
	Attend.NumberSlider.SetValue(Attend.raidsToKeepSlider, Attend.raidsToKeep)
	Attend.NumberSlider.SetValue(Attend.minRaidLevelSlider, Attend.minRaidLevel)
	Attend.configWindow:SetVisible(true)
end

function Attend.ConfirmationWindowClose(confirmed)
	Attend.confirmationWindow:SetVisible(false)
	Attend.confirmationCallback(confirmed)
end

function Attend.CreateConfirmationWindow(parentFrame, positonFrame)
	-- Create confirmation window
	Attend.confirmationWindow = UI.CreateFrame("SimpleWindow", "ConfirmationWindow", parentFrame)
	Attend.confirmationWindow:SetVisible(false)
	Attend.confirmationWindow:SetTitle("Confirmation window")
	Attend.confirmationWindow:SetHeight(150)
	Attend.confirmationWindow:SetPoint("CENTER", positonFrame, "CENTER")
	Attend.confirmationWindow:SetLayer(5)
	
	Attend.confirmationLabel = UI.CreateFrame("Text", "ConfirmationLabel", Attend.confirmationWindow)
	Attend.confirmationLabel:SetPoint("TOPLEFT", Attend.confirmationWindow, "TOPLEFT", 20, 60)
	Attend.confirmationLabel:SetPoint("TOPRIGHT", Attend.confirmationWindow, "TOPRIGHT", -20, 60)
	Attend.confirmationLabel:SetWordwrap(true)
	Attend.confirmationLabel:SetText("Dummy message")
	
	Attend.confirmationWindowYesButton = UI.CreateFrame("RiftButton", "ConfirmationYesButton", Attend.confirmationWindow)
	Attend.confirmationWindowYesButton:SetText("Yes")
	Attend.confirmationWindowYesButton:SetPoint("BOTTOMLEFT", Attend.confirmationWindow, "BOTTOMLEFT", 20, -20)
	Attend.confirmationWindowYesButton.Event.LeftPress = function() Attend.ConfirmationWindowClose(true) end
	
	Attend.confirmationWindowNoButton = UI.CreateFrame("RiftButton", "ConfirmationNoButton", Attend.confirmationWindow)
	Attend.confirmationWindowNoButton:SetText("No")
	Attend.confirmationWindowNoButton:SetPoint("BOTTOMRIGHT", Attend.confirmationWindow, "BOTTOMRIGHT", -20, -20)
	Attend.confirmationWindowNoButton.Event.LeftPress = function() Attend.ConfirmationWindowClose(false) end
	
	Attend.confirmationWindow:SetWidth(Attend.confirmationWindowYesButton:GetWidth() * 2 + 60)

	Attend.confirmationCallback = nil
end

function Attend.ShowConfirmationWindow(message, callback)
	Attend.confirmationCallback = callback
	Attend.confirmationLabel:SetText(message)
	Attend.confirmationWindow:SetVisible(true)
end

function Attend.ExportWindowClose()
	Attend.exportText:SetText("")
	Attend.exportText:SetKeyFocus(false)
	Attend.exportWindow:SetVisible(false)
	
	Attend.SetRaidDetailWindowEnabled(true)
end

function Attend.CreateExportWindow(parentFrame, positonFrame)
	-- Create export window
	Attend.exportWindow = UI.CreateFrame("SimpleWindow", "ExportFrame", parentFrame)
	Attend.exportWindow:SetVisible(false)
	Attend.exportWindow:SetWidth(200)
	Attend.exportWindow:SetHeight(150)
	Attend.exportWindow:SetPoint("CENTER", positonFrame, "CENTER")
	Attend.exportWindow:SetTitle("Export raid window")
	Attend.exportWindow:SetLayer(5)
	
	Attend.exportText = UI.CreateFrame("RiftTextfield", "ExportText", Attend.exportWindow)
	Attend.exportText:SetPoint("TOPLEFT", Attend.exportWindow, "TOPLEFT", 20, 60)
	Attend.exportText:SetPoint("TOPRIGHT", Attend.exportWindow, "TOPRIGHT", -20, 60)
	
	Attend.exportLabel = UI.CreateFrame("Text", "ExportLabel", Attend.exportWindow)
	Attend.exportLabel:SetPoint("TOPLEFT", Attend.exportWindow, "TOPLEFT", 20, 90)
	Attend.exportLabel:SetPoint("TOPRIGHT", Attend.exportWindow, "TOPRIGHT", -20, 90)
	Attend.exportLabel:SetText("Press Ctrl+C to copy raid")
	
	Attend.exportWindowButton = UI.CreateFrame("RiftButton", "ExportButton", Attend.exportWindow)
	Attend.exportWindowButton:SetText("OK")
	Attend.exportWindowButton:SetPoint("TOPLEFT", Attend.exportWindow, "BOTTOMLEFT", 40, -35)
	Attend.exportWindowButton.Event.LeftPress = Attend.ExportWindowClose
end

function Attend.ShowExportWindow(textString)
	Attend.exportWindow:SetVisible(true)
	Attend.exportText:SetText(textString)
	Attend.exportText:SetKeyFocus(true)
	Attend.exportText:SetSelection(0, textString:len())
end

function Attend.StandbyWindowClose(addMember)
	Attend.standbyText:SetKeyFocus(false)
	Attend.standbyWindow:SetVisible(false)
	Attend.SetRaidDetailWindowEnabled(true)

	if addMember then
		Attend.AddStandbyPlayer(Attend.standbyText:GetText(), Attend.GetTime())
		Attend.ShowRaidDetailWindow(raidDetail)
	end
end

function Attend.CreateStandbyWindow(parentFrame, positonFrame)
	-- Create standby window
	Attend.standbyWindow = UI.CreateFrame("SimpleWindow", "StandbyFrame", parentFrame)
	Attend.standbyWindow:SetVisible(false)
	Attend.standbyWindow:SetHeight(180)
	Attend.standbyWindow:SetPoint("CENTER", positonFrame, "CENTER")
	Attend.standbyWindow:SetTitle("Add Standby raid member")
	Attend.standbyWindow:SetLayer(5)
	
	Attend.standbyLabel = UI.CreateFrame("Text", "StandbyLabel", Attend.standbyWindow)
	Attend.standbyLabel:SetPoint("TOPLEFT", Attend.standbyWindow, "TOPLEFT", 20, 60)
	Attend.standbyLabel:SetPoint("TOPRIGHT", Attend.standbyWindow, "TOPRIGHT", -20, 60)
	Attend.standbyLabel:SetText("Enter name below")

	Attend.standbyText = UI.CreateFrame("RiftTextfield", "StandbyText", Attend.standbyWindow)
	Attend.standbyText:SetPoint("TOPLEFT", Attend.standbyWindow, "TOPLEFT", 20, 90)
	Attend.standbyText:SetPoint("TOPRIGHT", Attend.standbyWindow, "TOPRIGHT", -20, 90)
	
	Attend.standbyWindowOKButton = UI.CreateFrame("RiftButton", "StandbyOKButton", Attend.standbyWindow)
	Attend.standbyWindowOKButton:SetText("OK")
	Attend.standbyWindowOKButton:SetPoint("BOTTOMLEFT", Attend.standbyWindow, "BOTTOMLEFT", 20, -20)
	Attend.standbyWindowOKButton.Event.LeftPress = function() Attend.StandbyWindowClose(true) end
	
	Attend.standbyWindowCancelButton = UI.CreateFrame("RiftButton", "StandbyCancelButton", Attend.standbyWindow)
	Attend.standbyWindowCancelButton:SetText("Cancel")
	Attend.standbyWindowCancelButton:SetPoint("BOTTOMRIGHT", Attend.standbyWindow, "BOTTOMRIGHT", -20, -20)
	Attend.standbyWindowCancelButton.Event.LeftPress = function() Attend.StandbyWindowClose(false) end
	
	Attend.standbyWindow:SetWidth(Attend.standbyWindowOKButton:GetWidth() * 2 + 60)
end

function Attend.ShowStandbyWindow()
	Attend.standbyWindow:SetVisible(true)
	Attend.standbyText:SetText("")
	Attend.standbyText:SetKeyFocus(true)
end

function Attend.LootAddWindowClose(okPressed, lootNumber)	
	Attend.lootItemText:SetKeyFocus(false)
	Attend.lootCostText:SetKeyFocus(false)
	Attend.lootBossText:SetKeyFocus(false)
	Attend.lootAddWindow:SetVisible(false)

	if okPressed then
		local boss = Attend.lootBossText:GetText()
		if boss == nil or boss == "" then
			boss = "boss"
		end
		
		if not lootNumber then
			Attend.AddLoot(raidDetail, raidPlayer, Attend.lootItemText:GetText(), Attend.GetNumber(Attend.lootCostText:GetText()), boss)
		else
			Attend.EditLoot(raidDetail, raidPlayer, lootNumber, Attend.lootItemText:GetText(), Attend.GetNumber(Attend.lootCostText:GetText()), boss)
		end
		
		raidItem = Attend.lootItemText:GetText()
	end
	
	Attend.ShowLootWindow(raidPlayer)	
end

function Attend.CreateLootAddWindow(parentFrame, positonFrame)
	-- Create lootAdd window
	Attend.lootAddWindow = UI.CreateFrame("SimpleWindow", "LootAddFrame", parentFrame)
	Attend.lootAddWindow:SetVisible(false)	
	Attend.lootAddWindow:SetWidth(400)
	Attend.lootAddWindow:SetHeight(400)
	Attend.lootAddWindow:SetPoint("CENTER", positonFrame, "CENTER")
	Attend.lootAddWindow:SetLayer(5)
	
	Attend.lootAddRowFrame = UI.CreateFrame("Frame", "LootAddAttend.RowLayoutFrame", Attend.lootAddWindow)
	Attend.lootAddRowFrame:SetPoint("TOPLEFT", Attend.lootAddWindow, "TOPLEFT", 50, 70)
	Attend.lootAddRowFrame:SetPoint("BOTTOMRIGHT", Attend.lootAddWindow, "BOTTOMRIGHT", -50, -70)
		
	Attend.lootAddAttendRowLayout = Attend.RowLayout.Create(Attend.lootAddRowFrame, 3, 2, 20, 70, 2, 2)
	Attend.RowLayout.SetColWidth(Attend.lootAddAttendRowLayout, 2, 200)

	local labelFrame = Attend.RowLayout.CreateFrame(Attend.lootAddAttendRowLayout, 1, 1, "Text")
	labelFrame:SetText("Item")
	labelFrame = Attend.RowLayout.CreateFrame(Attend.lootAddAttendRowLayout, 2, 1, "Text")
	labelFrame:SetText("Cost")
	labelFrame = Attend.RowLayout.CreateFrame(Attend.lootAddAttendRowLayout, 3, 1, "Text")
	labelFrame:SetText("Boss")

	Attend.lootItemText = Attend.RowLayout.CreateFrame(Attend.lootAddAttendRowLayout, 1, 2, "RiftTextfield")
	Attend.lootCostText = Attend.RowLayout.CreateFrame(Attend.lootAddAttendRowLayout, 2, 2, "RiftTextfield")
	Attend.lootBossText = Attend.RowLayout.CreateFrame(Attend.lootAddAttendRowLayout, 3, 2, "RiftTextfield")
	
	Attend.RowLayout.Redraw(Attend.lootAddAttendRowLayout)
	
	Attend.lootAddWindowOKButton = UI.CreateFrame("RiftButton", "LootAddOKButton", Attend.lootAddWindow)
	Attend.lootAddWindowOKButton:SetText("OK")
	Attend.lootAddWindowOKButton:SetPoint("BOTTOMLEFT", Attend.lootAddWindow, "BOTTOMLEFT", 20, -20)
	
	Attend.lootAddWindowCancelButton = UI.CreateFrame("RiftButton", "LootAddCancelButton", Attend.lootAddWindow)
	Attend.lootAddWindowCancelButton:SetText("Cancel")
	Attend.lootAddWindowCancelButton:SetPoint("BOTTOMRIGHT", Attend.lootAddWindow, "BOTTOMRIGHT", -20, -20)
	Attend.lootAddWindowCancelButton.Event.LeftPress = function() Attend.LootAddWindowClose(false) end
end

function Attend.ShowLootAddWindow(lootNumber)
	Attend.lootAddWindow:SetVisible(true)
	if lootNumber then
		Attend.lootAddWindowOKButton.Event.LeftPress = function() Attend.LootAddWindowClose(true, lootNumber) end
		
		Attend.lootAddWindow:SetTitle("Edit loot item")
		Attend.lootItemText:SetText(raidLoot[lootNumber].name)
		Attend.lootCostText:SetText(string.format("%d", raidLoot[lootNumber].cost))
		local boss = raidLoot[lootNumber].bossName
		if not boss then
			boss = ""
		end
		Attend.lootBossText:SetText(boss)
		raidItem = raidLoot[lootNumber].name
	else
		Attend.lootAddWindowOKButton.Event.LeftPress = function() Attend.LootAddWindowClose(true) end
		
		local bossName = ""
		local bossDeath = 0
		for name, death in pairs(Attend.GetBossKills(raidDetail)) do
			if death > bossDeath then
				bossName = name
				bossDeath = death
			end
		end
		
		Attend.lootAddWindow:SetTitle("Add loot item")
		Attend.lootItemText:SetText("")
		Attend.lootCostText:SetText("")
		Attend.lootBossText:SetText(bossName)
		raidItem = ""
	end
	
	Attend.lootItemText:SetKeyFocus(true)
end

function Attend.LootWindowClose(addMember)
	Attend.lootWindow:SetVisible(false)
	Attend.ClearLootWindow()
	Attend.ShowRaidDetailWindow(raidDetail)
end

function Attend.LootAddPressed()
	Attend.SetLootWindowEnabled(false)
	Attend.ShowLootAddWindow()
end

function Attend.LootEditPressed(lootNumber)
	Attend.SetLootWindowEnabled(false)
	Attend.ShowLootAddWindow(lootNumber)
end

function Attend.LootDeleteConfirmation(confirmed, lootNumber)
	if confirmed then
		Attend.RemoveLoot(raidDetail, raidPlayer, lootNumber)
	end
	
	Attend.ShowLootWindow(raidPlayer)
end

function Attend.LootDeletePressed(lootNumber)
	Attend.SetLootWindowEnabled(false)
	Attend.ShowConfirmationWindow("Really delete this item?", function(confirmed) Attend.LootDeleteConfirmation(confirmed, lootNumber) end)
end

function Attend.SetLootWindowEnabled(enabled)
	Attend.lootAddButton:SetEnabled(enabled)
	Attend.lootOKButton:SetEnabled(enabled)
	Attend.lootAddButton:SetText("Add")
	Attend.lootOKButton:SetText("OK")
	
	if Attend.lootAttendRowLayout then
		for indx = 2, Attend.RowLayout.GetNumRows(Attend.lootAttendRowLayout) do
			local buttonFrame = Attend.RowLayout.GetCellFrame(Attend.lootAttendRowLayout, indx, 4)
			buttonFrame:SetEnabled(enabled)
			buttonFrame:SetText("Edit")
			buttonFrame:SetHeight(Attend.RowLayout.GetRowHeight(Attend.lootAttendRowLayout, indx))

			buttonFrame = Attend.RowLayout.GetCellFrame(Attend.lootAttendRowLayout, indx, 5)
			buttonFrame:SetEnabled(enabled)
			buttonFrame:SetText("Delete")
			buttonFrame:SetHeight(Attend.RowLayout.GetRowHeight(Attend.lootAttendRowLayout, indx))
		end
	end
end

function Attend.ShowLootWindow(thisPlayer)
	raidLoot = Attend.GetPlayerLoot(raidDetail, thisPlayer)
	raidPlayer = thisPlayer
	
	Attend.ClearLootWindow()

	Attend.lootWindow:SetVisible(true)
	
	local rowMask = UI.CreateFrame("Mask", "LootAttend.RowLayoutMask", Attend.lootWindow)
	rowMask:SetPoint("TOPLEFT", Attend.lootWindow, "TOPLEFT", 50, 70)
	rowMask:SetPoint("BOTTOMRIGHT", Attend.lootWindow, "BOTTOMRIGHT", -50, -70)
	
	local rowFrame = UI.CreateFrame("Frame", "LootAttend.RowLayout", rowMask)
	rowFrame:SetAllPoints(rowMask)
	
	Attend.lootAttendRowLayout = Attend.RowLayout.Create(rowFrame, #raidLoot + 1, 5, 20, 70, 2, 2)
	Attend.RowLayout.SetColWidth(Attend.lootAttendRowLayout, 1, 200)

	local titleFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, 1, 1, "Text")
	titleFrame:SetText("NAME")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	titleFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, 1, 2, "Text")
	titleFrame:SetText("Cost")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	titleFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, 1, 3, "Text")
	titleFrame:SetText("Boss")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	titleFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, 1, 4, "Text")
	titleFrame:SetText("Edit")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	titleFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, 1, 5, "Text")
	titleFrame:SetText("Delete")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	
	for row, loot in ipairs(raidLoot) do
		local nameFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, row + 1, 1, "Text")
		nameFrame:SetText(loot.name)
		
		local costFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, row + 1, 2, "Text")
		costFrame:SetText(string.format("%d", loot.cost))

		local bossFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, row + 1, 3, "Text")
		local bossText = loot.bossName
		if not bossText then
			bossText = ""
		end
		bossFrame:SetText(bossText)

		local buttonFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, row + 1, 4, "RiftButton")
		buttonFrame.Event.LeftPress = function() Attend.LootEditPressed(row) end

		buttonFrame = Attend.RowLayout.CreateFrame(Attend.lootAttendRowLayout, row + 1, 5, "RiftButton")
		buttonFrame.Event.LeftPress = function() Attend.LootDeletePressed(row) end
	end
	
	Attend.RowLayout.Redraw(Attend.lootAttendRowLayout)
	Attend.SetLootWindowEnabled(true)
end

function Attend.ClearLootWindow()
	if Attend.lootAttendRowLayout then
		Attend.RowLayout.GetFrame(Attend.lootAttendRowLayout):SetVisible(false)
		Attend.lootAttendRowLayout = nil
	end
end

function Attend.CreateLootWindow(parentFrame)
	Attend.lootWindow = UI.CreateFrame("SimpleWindow", "LootFrame", parentFrame)
	Attend.lootWindow:SetVisible(false)
	Attend.lootWindow:SetAllPoints(Attend.raidListWindow)
	Attend.lootWindow:SetTitle("Player Loot")

	Attend.lootAddButton = UI.CreateFrame("RiftButton", "LootAddButton", Attend.lootWindow)
	Attend.lootAddButton:SetPoint("BOTTOMRIGHT", Attend.lootWindow, "BOTTOMRIGHT", -20, -20)
	Attend.lootAddButton.Event.LeftPress = Attend.LootAddPressed

	Attend.lootOKButton = UI.CreateFrame("RiftButton", "LootOKButton", Attend.lootWindow)
	Attend.lootOKButton:SetPoint("BOTTOMLEFT", Attend.lootWindow, "BOTTOMLEFT", 20, -20)
	Attend.lootOKButton.Event.LeftPress = Attend.LootWindowClose	
end

function Attend.CreatePointAddWindow(parentFrame)
	Attend.ptAddWindow = UI.CreateFrame("SimpleWindow", "PointAddFrame", parentFrame)
	Attend.ptAddWindow:SetVisible(false)
	Attend.ptAddWindow:SetAllPoints(Attend.raidListWindow)
	Attend.ptAddWindow:SetTitle("Add Player Points")
	
	local row = 6
	local col1 = 20
	local col2 = 100
	
	local border = UI.CreateFrame("SimpleBorderExt", Attend.ptAddWindow:GetName() .. "RaidBorder", Attend.ptAddWindow)
	border:SetPoint("TOPLEFT", Attend.ptAddWindow, "TOPLEFT", col1, row * 30)
	border:SetPoint("BOTTOMRIGHT", Attend.ptAddWindow, "TOPRIGHT", -col1, row * 30 + 20 * 12)
	
	local frame = border:GetContent()
	local width = frame:GetWidth() / 2
	Attend.ptRaidMembers = {}
	for i = 1, 10 do
		local check = UI.CreateFrame("SimpleCheckbox", Attend.ptAddWindow:GetName() .. "RaidCheck" .. i, frame)
		check:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, (i - 1) * 20)
		Attend.ptRaidMembers[i] = check

		local check = UI.CreateFrame("SimpleCheckbox", Attend.ptAddWindow:GetName() .. "RaidCheck" .. (i + 10), frame)
		check:SetPoint("TOPLEFT", frame, "TOPLEFT", width, (i - 1) * 20)
		Attend.ptRaidMembers[i + 10] = check
	end

	row = row - 1
	local textFrame = UI.CreateFrame("Text", Attend.ptAddWindow:GetName() .. "PointsLabel", Attend.ptAddWindow)
	textFrame:SetText("Points")
	textFrame:SetPoint("TOPLEFT", Attend.ptAddWindow, "TOPLEFT", col1, row * 30)
	
	Attend.ptPoints = UI.CreateFrame("SimpleSlider", Attend.ptAddWindow:GetName() .. "Points", Attend.ptAddWindow)
	Attend.ptPoints:SetPoint("TOPLEFT", Attend.ptAddWindow, "TOPLEFT", col2, row * 30)
	Attend.ptPoints:SetRange(1, 500)

	row = row - 1
	local textFrame = UI.CreateFrame("Text", Attend.ptAddWindow:GetName() .. "DescLabel", Attend.ptAddWindow)
	textFrame:SetText("Description")
	textFrame:SetPoint("TOPLEFT", Attend.ptAddWindow, "TOPLEFT", col1, row * 30)
	
	Attend.ptDesc = UI.CreateFrame("RiftTextfield", Attend.ptAddWindow:GetName() .. "Desc", Attend.ptAddWindow)
	Attend.ptDesc:SetPoint("TOPLEFT", Attend.ptAddWindow, "TOPLEFT", col2, row * 30)

	row = row - 1
	local textFrame = UI.CreateFrame("Text", Attend.ptAddWindow:GetName() .. "SelectLabel", Attend.ptAddWindow)
	textFrame:SetText("Type")
	textFrame:SetPoint("TOPLEFT", Attend.ptAddWindow, "TOPLEFT", col1, row * 30)
	
	Attend.ptAddSelect = UI.CreateFrame("SimpleSelectExt", Attend.ptAddWindow:GetName() .. "Select", Attend.ptAddWindow)
	Attend.ptAddSelect:SetItems({"Boss Kill", "Arrived Early", "Hourly Bonus", "Other"})
	Attend.ptAddSelect:SetPoint("TOPLEFT", Attend.ptAddWindow, "TOPLEFT", col2, row * 30)
	Attend.ptAddSelect:SetHeight(textFrame:GetHeight())
	Attend.ptAddSelect:SetWidth(120)

end

function Attend.SetRaidDetailWindowEnabled(enabled)
	Attend.raidDetailExportBBButton:SetEnabled(enabled)
	Attend.raidDetailExportXMLButton:SetEnabled(enabled)
	Attend.raidDetailOKButton:SetEnabled(enabled)
	Attend.raidDetailMergeButton:SetEnabled(enabled)
	Attend.raidDetailStandbyButton:SetEnabled(enabled)
	Attend.raidDetailExportBBButton:SetText("Export")
	Attend.raidDetailExportXMLButton:SetText("Export XML")
	Attend.raidDetailOKButton:SetText("OK")
	Attend.raidDetailMergeButton:SetText("Merge")
	Attend.raidDetailStandbyButton:SetText("Standby")
	
	if Attend.raidDetailRowLayout then
		for indx = 2, Attend.RowLayout.GetNumRows(Attend.raidDetailRowLayout) do
			local buttonFrame = Attend.RowLayout.GetCellFrame(Attend.raidDetailRowLayout, indx, 4)
			buttonFrame:SetEnabled(enabled)
			buttonFrame:SetText("Open")
			buttonFrame:SetHeight(Attend.RowLayout.GetRowHeight(Attend.raidDetailRowLayout, indx))
		end
	end
end

function Attend.RaidDetailMergeConfirmed(confirmed)
	if confirmed then
		local previousRaid = nil
		local detailIndex = 0
		for indx = 2, #raids do
			if raids[indx] == raidDetail then
				previousRaid = raids[indx-1]
				detailIndex = indx
				break
			end
		end
		
		if previousRaid then
			raids[detailIndex - 1] = Attend.MergeRaids(previousRaid, raidDetail)
			table.remove(raids, detailIndex)
		end

		Attend.SetRaidDetailWindowEnabled(true)
		Attend.RaidDetailWindowClose()
		Attend.ShowRaidListWindow()
	else
		Attend.SetRaidDetailWindowEnabled(true)
	end
end

function Attend.RaidDetailMerge()
	Attend.SetRaidDetailWindowEnabled(false)
	Attend.ShowConfirmationWindow("Are you sure you wish to merge this raid with the previous raid?", Attend.RaidDetailMergeConfirmed)
end

function Attend.RaidDetailStandby()
	Attend.SetRaidDetailWindowEnabled(false)
	Attend.ShowStandbyWindow()
end

function Attend.RaidDetailExportBB()
	Attend.SetRaidDetailWindowEnabled(false)
	Attend.ShowExportWindow(Attend.GetBBCodeExport(raidDetail))
end

function Attend.RaidDetailExportXML()
	Attend.SetRaidDetailWindowEnabled(false)
	Attend.ShowExportWindow(Attend.GetXMLExport(raidDetail))
end

function Attend.RaidDetailWindowClose()
	Attend.raidDetailWindow:SetVisible(false)
	Attend.ClearRaidDetailWindow()
	Attend.ShowRaidListWindow()
end

function Attend.RaidDetailLootPressed(thisPlayer)
	if thisPlayer then
		Attend.raidDetailWindow:SetVisible(false)
		Attend.ClearRaidDetailWindow()
		Attend.ShowLootWindow(thisPlayer)
	end
end

function Attend.ClearRaidDetailWindow()
	if Attend.raidDetailRowLayout then
		Attend.RowLayout.GetFrame(Attend.raidDetailRowLayout):SetVisible(false)
		Attend.raidDetailRowLayout = nil
	end
end

function Attend.ShowRaidDetailWindow(thisRaid)
	raidDetail = thisRaid
	
	if raidDetail == raids[1] or not raidDetail.endTime then
		Attend.raidDetailMergeButton:SetVisible(false)
	else
		Attend.raidDetailMergeButton:SetVisible(true)
	end
	
	if not raidDetail.endTime then
		Attend.raidDetailStandbyButton:SetVisible(true)
	else
		Attend.raidDetailStandbyButton:SetVisible(false)
	end
	
	Attend.ClearRaidDetailWindow()

	Attend.raidDetailWindow:SetVisible(true)
	
	local rowMask = UI.CreateFrame("Mask", "RaidDetailAttend.RowLayoutMask", Attend.raidDetailWindow)
	rowMask:SetPoint("TOPLEFT", Attend.raidDetailWindow, "TOPLEFT", 50, 70)
	rowMask:SetPoint("BOTTOMRIGHT", Attend.raidDetailWindow, "BOTTOMRIGHT", -50, -70)
	
	local rowFrame = UI.CreateFrame("Frame", "RaidDetailAttend.RowLayout", rowMask)
	rowFrame:SetAllPoints(rowMask)
	
	local raidDesc, playerNames = Attend.GetRaidDescription(thisRaid)

	Attend.raidDetailRowLayout = Attend.RowLayout.Create(rowFrame, #playerNames + 1, 4, 20, 70, 2, 2)
	Attend.RowLayout.SetColWidth(Attend.raidDetailRowLayout, 1, 200)

	local titleFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, 1, 1, "Text")
	titleFrame:SetText("NAME")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	titleFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, 1, 2, "Text")
	titleFrame:SetText("Duration")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	titleFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, 1, 3, "Text")
	titleFrame:SetText("Standby")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	titleFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, 1, 4, "Text")
	titleFrame:SetText("Loot")
	titleFrame:SetFontSize(titleFrame:GetFontSize() + 2)
	
	for row = 1, #playerNames do
		local nameFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, row + 1, 1, "Text")
		nameFrame:SetText(playerNames[row])
		
		local durationFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, row + 1, 2, "Text")
		durationFrame:SetText(Attend.FormatTime(thisRaid.players[playerNames[row]].duration))

		if thisRaid.players[playerNames[row]].standbyDuration > 0 then
			local standbyFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, row + 1, 3, "Text")
			standbyFrame:SetText(Attend.FormatTime(thisRaid.players[playerNames[row]].standbyDuration))
		end
		
		local buttonFrame = Attend.RowLayout.CreateFrame(Attend.raidDetailRowLayout, row + 1, 4, "RiftButton")
		buttonFrame.Event.LeftPress = function() Attend.RaidDetailLootPressed(playerNames[row]) end
	end
	
	Attend.RowLayout.Redraw(Attend.raidDetailRowLayout)
	Attend.SetRaidDetailWindowEnabled(true)
end

function Attend.CreateRaidDetailWindow(parentFrame)
	Attend.raidDetailWindow = UI.CreateFrame("SimpleWindow", "RaidDetailFrame", parentFrame)
	Attend.raidDetailWindow:SetVisible(false)
	Attend.raidDetailWindow:SetAllPoints(Attend.raidListWindow)
	Attend.raidDetailWindow:SetTitle("Raid Attendance Detail")

	Attend.raidDetailOKButton = UI.CreateFrame("RiftButton", "RaidDetailOKButton", Attend.raidDetailWindow)
	Attend.raidDetailOKButton:SetPoint("BOTTOMLEFT", Attend.raidDetailWindow, "BOTTOMLEFT", 20, -20)
	Attend.raidDetailOKButton.Event.LeftPress = Attend.RaidDetailWindowClose
	
	Attend.raidDetailExportBBButton = UI.CreateFrame("RiftButton", "raidDetailExportBBButton", Attend.raidDetailWindow)
	Attend.raidDetailExportBBButton:SetPoint("BOTTOMLEFT", Attend.raidDetailOKButton, "BOTTOMRIGHT", 20, 0)
	Attend.raidDetailExportBBButton.Event.LeftPress = Attend.RaidDetailExportBB
	
	Attend.raidDetailExportXMLButton = UI.CreateFrame("RiftButton", "raidDetailExportXMLButton", Attend.raidDetailWindow)
	Attend.raidDetailExportXMLButton:SetPoint("BOTTOMLEFT", Attend.raidDetailExportBBButton, "BOTTOMRIGHT", 20, 0)
	Attend.raidDetailExportXMLButton.Event.LeftPress = Attend.RaidDetailExportXML
	
	Attend.raidDetailMergeButton = UI.CreateFrame("RiftButton", "RaidDetailMergeButton", Attend.raidDetailWindow)
	Attend.raidDetailMergeButton:SetPoint("BOTTOMLEFT", Attend.raidDetailExportXMLButton, "BOTTOMRIGHT", 20, 0)
	Attend.raidDetailMergeButton.Event.LeftPress = Attend.RaidDetailMerge
	
	Attend.raidDetailStandbyButton = UI.CreateFrame("RiftButton", "RaidDetailMergeButton", Attend.raidDetailWindow)
	Attend.raidDetailStandbyButton:SetPoint("BOTTOMLEFT", Attend.raidDetailMergeButton, "BOTTOMRIGHT", 20, 0)
	Attend.raidDetailStandbyButton.Event.LeftPress = Attend.RaidDetailStandby
	
	Attend.SetRaidDetailWindowEnabled(true)

	Attend.CreateLootWindow(parentFrame)
end

function Attend.RaidListWindowOpenPressed(thisRaid)
	if thisRaid then
		Attend.RaidListWindowClose()
		Attend.ShowRaidDetailWindow(thisRaid)
	end
end

function Attend.RaidListWindowEndConfirmed(confirmed)
	if confirmed then
		Attend.EndRaid(Attend.GetTime())
	end
	
	raidToEnd = nil
	Attend.ShowRaidListWindow()
end

function Attend.RaidListWindowEndPressed(thisRaid)
	raidToEnd = thisRaid
	Attend.SetRaidListWindowEnabled(false)
	Attend.ShowConfirmationWindow("Are you sure you wish to end this raid?", Attend.RaidListWindowEndConfirmed)
end

function Attend.RaidListWindowDeleteConfirmed(confirmed)
	if confirmed then
		Attend.DeleteRaid(raidToDelete)
	end
	
	raidToDelete = nil
	Attend.ShowRaidListWindow()
end

function Attend.RaidListWindowDeletePressed(thisRaid)
	raidToDelete = thisRaid
	Attend.SetRaidListWindowEnabled(false)
	Attend.ShowConfirmationWindow("Are you sure you wish to delete this raid?", Attend.RaidListWindowDeleteConfirmed)
end

function Attend.RaidListWindowClose()
	Attend.raidListWindow:SetVisible(false)
	
	local left, top, right, bottom = Attend.raidListWindow:GetBounds()
	Attend.x = left
	Attend.y = top
	
	Attend.ClearRaidListWindow()
end

function Attend.SetRaidListWindowEnabled(enabled)
	Attend.raidListOKButton:SetEnabled(enabled)
	Attend.raidListOKButton:SetText("OK")
	Attend.raidListConfigButton:SetEnabled(enabled)
	Attend.raidListConfigButton:SetText("Config")
	
	if Attend.raidListRowLayout then
		for indx = 1, Attend.RowLayout.GetNumRows(Attend.raidListRowLayout) do
			local buttonFrame = Attend.RowLayout.GetCellFrame(Attend.raidListRowLayout, indx, 3)
			buttonFrame:SetEnabled(enabled)
			
			buttonFrame = Attend.RowLayout.GetCellFrame(Attend.raidListRowLayout, indx, 4)
			buttonFrame:SetEnabled(enabled)
		end
	end
end

function Attend.CreateRaidListWindow(parentFrame)
	-- Check the saved coordinates are actually on the screen
	local left, top, right, bottom = UIParent:GetBounds()
	if Attend.x < left or Attend.x > right or Attend.y < top or Attend.y > bottom then
		Attend.x = 200
		Attend.y = 200
	end
	
	-- Create raid list window
	Attend.raidListWindow = UI.CreateFrame("SimpleWindow", "RaidListFrame", parentFrame)
	Attend.raidListWindow:SetVisible(false)
	Attend.raidListWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Attend.x, Attend.y)
	Attend.raidListWindow:SetHeight(800)
	Attend.raidListWindow:SetTitle("Raid Attendance Tracker")
	
	Attend.raidListOKButton = UI.CreateFrame("RiftButton", "RaidListOKButton", Attend.raidListWindow)
	Attend.raidListOKButton.Event.LeftPress = Attend.RaidListWindowClose

	Attend.raidListWindow:SetWidth(Attend.raidListOKButton:GetWidth() * 4 + 100)
	Attend.raidListOKButton:SetPoint("BOTTOMLEFT", Attend.raidListWindow, "BOTTOMLEFT", 20, -20)
	
	Attend.raidListConfigButton = UI.CreateFrame("RiftButton", "RaidListConfigButton", Attend.raidListWindow)
	Attend.raidListConfigButton.Event.LeftPress = Attend.ShowConfigWindow
	Attend.raidListConfigButton:SetPoint("BOTTOMRIGHT", Attend.raidListWindow, "BOTTOMRIGHT", -20, -20)
	
	Attend.SetRaidListWindowEnabled(true)

	Attend.CreateRaidDetailWindow(parentFrame)
end

function Attend.AddRaidListRow(thisRaid, row, isCurrent)
	local raidDesc, playerNames = Attend.GetRaidDescription(thisRaid)
	local textFrame = Attend.RowLayout.CreateFrame(Attend.raidListRowLayout, row, 1, "Text")
	textFrame:SetText(raidDesc)
	
	local nameFrame = Attend.RowLayout.CreateFrame(Attend.raidListRowLayout, row, 2, "Text")
	nameFrame:SetText(thisRaid.playerName)

	local buttonFrame = Attend.RowLayout.CreateFrame(Attend.raidListRowLayout, row, 3, "RiftButton")
	buttonFrame:SetText("Open")
	buttonFrame.Event.LeftPress = function() Attend.RaidListWindowOpenPressed(thisRaid) end

	buttonFrame = Attend.RowLayout.CreateFrame(Attend.raidListRowLayout, row, 4, "RiftButton")
	if isCurrent == true then
		buttonFrame:SetText("End")
		buttonFrame.Event.LeftPress = function() Attend.RaidListWindowEndPressed(thisRaid) end	
	else
		buttonFrame:SetText("Delete")
		buttonFrame.Event.LeftPress = function() Attend.RaidListWindowDeletePressed(thisRaid) end
	end
end

function Attend.ClearRaidListWindow()
	if Attend.raidListRowLayout then
		Attend.RowLayout.GetFrame(Attend.raidListRowLayout):SetVisible(false)
		Attend.raidListRowLayout = nil
	end
end

function Attend.ShowRaidListWindow()
	local numberOfRaids = #raids
	if Attend.IsInRaid() then
		numberOfRaids = numberOfRaids + 1
	end
	
	Attend.ClearRaidListWindow()

	local rowMask = UI.CreateFrame("Frame", "RaidListAttend.RowLayoutMask", Attend.raidListWindow)
	rowMask:SetPoint("TOPLEFT", Attend.raidListWindow, "TOPLEFT", 30, 70)
	rowMask:SetPoint("BOTTOMRIGHT", Attend.raidListWindow, "BOTTOMRIGHT", -50, -70)
	
	local rowFrame = UI.CreateFrame("Frame", "RaidListAttend.RowLayout", rowMask)
	rowFrame:SetAllPoints(rowMask)

	raidWindowButtonMap = {}
	
	if numberOfRaids > 0 then
		Attend.raidListRowLayout = Attend.RowLayout.Create(rowFrame, numberOfRaids, 4, 20, 70, 2, 2)
		Attend.RowLayout.SetColWidth(Attend.raidListRowLayout, 1, 350)
		Attend.RowLayout.SetColWidth(Attend.raidListRowLayout, 2, 70)

		for row = 1, #raids do
			Attend.AddRaidListRow(raids[row], row, false)
		end

		if Attend.IsInRaid() then
			Attend.AddRaidListRow(currentRaid, numberOfRaids, true)
		end

		Attend.RowLayout.Redraw(Attend.raidListRowLayout)
	else
		local miscFrame = UI.CreateFrame("Text", "MiscFrame", rowFrame)
		miscFrame:SetAllPoints(rowFrame)
		miscFrame:SetText("No raids stored")
	end
	
	Attend.SetRaidListWindowEnabled(true)
	Attend.raidListWindow:SetVisible(true)
end

function Attend.OpenFrame()
	Attend.ShowRaidListWindow()
end

function Attend.CloseFrame()
	Attend.raidListWindow:SetVisible(false)
end

function Attend.ToggleFrame()
	if Attend.raidDetailWindow:GetVisible() == true or
	   Attend.raidListWindow:GetVisible() == true or
	   Attend.lootWindow:GetVisible() == true or
	   Attend.lootAddWindow:GetVisible() == true then
		Attend.raidDetailWindow:SetVisible(false)
		Attend.raidListWindow:SetVisible(false)
		Attend.lootWindow:SetVisible(false)
		Attend.lootAddWindow:SetVisible(false)
		Attend.exportWindow:SetVisible(false)
		Attend.configWindow:SetVisible(false)
		Attend.standbyWindow:SetVisible(false)
	elseif Attend.IsInRaid() then
		Attend.RaidListWindowOpenPressed(currentRaid)
	else
		Attend.ShowRaidListWindow()
	end
	
end

function Attend.CreateUI()
	local parent = UI.CreateContext("AttendContext")
	local parentFrame = UI.CreateFrame("Frame", "blah", parent);
	parentFrame:SetPoint("CENTER", UIParent, "CENTER")
	Attend.context = parent
	Attend.CreateRaidListWindow(parentFrame)
	Attend.CreateConfirmationWindow(parentFrame, Attend.raidListWindow)
	Attend.CreateExportWindow(parentFrame, Attend.raidListWindow)
	Attend.CreateStandbyWindow(parentFrame, Attend.raidListWindow)
	Attend.CreateConfigWindow(parentFrame, Attend.raidListWindow)
	Attend.CreateLootAddWindow(parentFrame, Attend.raidListWindow)
	Attend.CreatePointAddWindow(parentFrame, Attend.raidListWindow)
end

--
-- Event Handlers
--

function Attend.Initialise(h, addon)
	if addon == Attend.name then
		Attend.CreateUI()
		Attend.startup = true
	end
end

function Attend.CreateButton()
	-- create the button	
	local x = Attend.buttonX
	local y = Attend.buttonY
	Attend.button = UI.CreateFrame("Texture", "AttendButton", Attend.context)
	Attend.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	Attend.button:SetTexture("Attend", "user_group.png")
	Attend.button.Event.LeftClick = function() Attend.ToggleFrame() end
	Attend.DragFrame.Create(Attend.button, Attend.button:GetWidth(), Attend.button:GetHeight(), function(dragFrame) Attend.SetCoords(dragFrame.x, dragFrame.y) end)
end

function Attend.SetCoords(x, y)
	Attend.buttonX = x
	Attend.buttonY = y
	Attend.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
end

function Attend.Startup()
	if Attend.startup == true then
		Attend.startup = false
		
		local playerDetails = Inspect.Unit.Detail("player")
		Attend.playerName = playerDetails.name
		
		Attend.CreateEmptyRaid()
		Attend.Refresh()

		Attend.RegisterPostStartupEvents()
		
		Attend.CreateButton()

		Command.Event.Detach(Event.System.Update.Begin, Attend.Startup, "Attend.Startup")
	end
end

function Attend.EventHandler(h, eventTable)
	local refresh = 0
	local dumpIt = 0
	for unitID, unitSpecifier in pairs(eventTable) do
		if unitSpecifier == "player" then
			refresh = 1
		elseif type(unitSpecifier) == "string" and string.find(unitSpecifier, "group") then
			refresh = 1
		elseif type(unitSpecifier) == "string" and string.find(unitSpecifier, "mouseover") == nil and string.find(unitSpecifier, "target") == nil and string.find(unitSpecifier, "focus") == nil then
			dumpIt = 1
		end
	end
	
	if refresh == 1 then
		Attend.Refresh()
	else
		if dumpIt == 1 then
			dump(eventTable)
		end
	end
end

function Attend.SlashHandler(h, args)
	local arg = Attend.SplitArgs(args)
	
	if arg[1] == "dump" then
		Attend.Dump()
	elseif arg[1] == "standby" then
		print(Attend.AddStandbyPlayer(arg[2], Attend.GetTime()))
	elseif arg[1] == "loot" then
		print(Attend.AddPlayerLoot(args))
	elseif arg[1] == "open" or not arg[1] then
		Attend.OpenFrame()
	elseif arg[1] == "point" then
		Attend.ptAddWindow:SetVisible(true)
	else
		print(string.format("usage: %s [ open | dump | standby ]", Attend.slashName));
		print(string.format("   %s open", Attend.slashName));
		print(string.format("   %s dump", Attend.slashName));
		print(string.format("   %s standby playername", Attend.slashName));
	end
end

function Attend.SaveVariables(h, addon)
	if addon == Attend.name then
		-- end the current raid before saving
		Attend.EndRaid(Attend.GetTime())

		-- now copy saved group to settings so that they can be preserved on logout
		Attend_SavedVariables = {}
		Attend_SavedVariables.version = Attend.version
		Attend_SavedVariables.debug = Attend.debug
		Attend_SavedVariables.raidPlayerLimit = Attend.raidPlayerLimit
		Attend_SavedVariables.raidsToKeep = Attend.raidsToKeep
		Attend_SavedVariables.raids = raids
		Attend_SavedVariables.x = Attend.x
		Attend_SavedVariables.y = Attend.y
		Attend_SavedVariables.buttonX = Attend.buttonX
		Attend_SavedVariables.buttonY = Attend.buttonY
		Attend_SavedVariables.ignoreCrossShardRaids = Attend.ignoreCrossShardRaids
		Attend_SavedVariables.minRaidLevel = Attend.minRaidLevel
	end	
end

function Attend.GetSavedValue(value, default)
	if value ~= nil then
		return value
	else
		return default
	end
end

local function isPositiveInteger(n)
  return type(n) == "number" and n > 0 and math.floor(n) == n
end

local function isArray(array)
  local max, n = 0, 0
  for k, _ in pairs(array) do
    if not isPositiveInteger(k) then return false end
    max = math.max(max, k)
    n = n + 1
  end
  return n == max
end

function Attend.LoadVariables(h, addon)
	if addon == Attend.name then
		-- now that variables are loaded and saved positions restored we can create frame
		if Attend_SavedVariables then
			Attend.debug = Attend.GetSavedValue(Attend_SavedVariables.debug, false)
			Attend.raidPlayerLimit = Attend.GetSavedValue(Attend_SavedVariables.raidPlayerLimit, 8)
			Attend.raidsToKeep = Attend.GetSavedValue(Attend_SavedVariables.raidsToKeep, 20)
			raids = Attend.GetSavedValue(Attend_SavedVariables.raids, {})
			Attend.x = Attend.GetSavedValue(Attend_SavedVariables.x, 200)
			Attend.y = Attend.GetSavedValue(Attend_SavedVariables.y, 200)
			Attend.buttonX = Attend.GetSavedValue(Attend_SavedVariables.buttonX, 200)
			Attend.buttonY = Attend.GetSavedValue(Attend_SavedVariables.buttonY, 200)
			Attend.ignoreCrossShardRaids = Attend.GetSavedValue(Attend_SavedVariables.ignoreCrossShardRaids, false)
			Attend.minRaidLevel = Attend.GetSavedValue(Attend_SavedVariables.minRaidLevel, Attend.maxLevel)
			
			if Attend_SavedVariables.version ~= Attend.version then
				for _, raid in ipairs(raids) do
					for name, details in pairs(raid.players) do
						if isArray(details.loot) == false then
							local newLoot = {}
							for item, loot in pairs(details.loot) do
								table.insert(newLoot, loot)
							end
							
							details.loot = newLoot
						end
					end
				end
			end

			local savedVersion = tonumber(Attend_SavedVariables.version)
			if savedVersion == nil or savedVersion < 2.1 then
				Attend.Debug("Reset ignoreCrossShardRaids")
				Attend.ignoreCrossShardRaids = false
			end
			
			Attend.CreateEmptyRaid()
		end
	end	
end

function Attend.UnitDied(h, info)
	if Attend.IsInRaid() then
		if info.targetName ~= nil and Attend.raidZoneMap[info.targetName] ~= nil then
			currentRaid.location = Attend.raidZoneMap[info.targetName]
			local bossKills = Attend.GetBossKills(currentRaid)
			bossKills[info.targetName] = Attend.GetTime()
		end
	end
end

--
-- Register events
--
function Attend.RegisterPostStartupEvents()
	Command.Event.Attach(Command.Slash.Register(Attend.slashName), Attend.SlashHandler, "Attend.SlashHandler")
	Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, Attend.SaveVariables, "Attend.SaveVariables")
	Command.Event.Attach(Event.Unit.Add, Attend.EventHandler, "Attend.EventHandler")
	Command.Event.Attach(Event.Unit.Remove, Attend.EventHandler, "Attend.EventHandler")
	Command.Event.Attach(Event.Combat.Death, Attend.UnitDied, "Attend.UnitDied")
end

Command.Event.Attach(Event.Addon.Load.End, Attend.Initialise, "Attend.Initialise")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, Attend.LoadVariables, "Attend.LoadVariables")
Command.Event.Attach(Event.System.Update.Begin, Attend.Startup, "Attend.Startup")
